' -----[ Title ]------------------------------------------------------
'
' File...... LCDDEMO2.BAS
' Purpose... Stamp -> LCD (4-bit interface) with custom characters
' Author.... Jon Williams
' Started... 16 July 1994
' Updated... 16 July 1994


' -----[ Program Description ]----------------------------------------
'
' This program demonstrates the generation of custom characters for an
' LCD display that uses the Hitatchi HD44780 controller.  The LCD used 
' to test this program was the Optrex DMC-16106 (16x1).
'
' LCD Connections:
'
' LCD        (Function)         Stamp
' ---------------------         -----
' pin 1         Vss             gnd
' pin 2         Vdd             +5
' pin 3         Vo              gnd
' pin 4         RS              pin 4
' pin 5         R/W             gnd
' pin 6         E               pin 5
' pin 7         DB0             gnd
' pin 8         DB1             gnd
' pin 9         DB2             gnd
' pin 10        DB3             gnd
' pin 11        DB4             pin 0
' pin 12        DB5             pin 1
' pin 13        DB6             pin 2
' pin 14        DB7             pin 4


' -----[ Revision History ]-------------------------------------------
'
' 07-16-94 : Version 1.0


' -----[ Constants ]--------------------------------------------------
'
SYMBOL  E       = 5                     ' LCD enable pin (1 = enabled)
SYMBOL  RS      = 4                     ' Register Select (1 = char)

' LCD control characters
'
SYMBOL  ClrLCD  = $01                   ' clear the LCD
SYMBOL  CrsrHm  = $02                   ' move cursor to home position
SYMBOL  CrsrLf  = $10                   ' move cursor left
SYMBOL  CrsrRt  = $14                   ' move cursor right
SYMBOL  DispLf  = $18                   ' shift displayed chars left
SYMBOL  DispRt  = $1C                   ' shift displayed chars right

 
' -----[ Variables ]--------------------------------------------------
'
SYMBOL  outp    = B0                    ' output workspace
SYMBOL  char    = B1                    ' char sent to LCD
SYMBOL  index1  = B2                    ' loop counter
SYMBOL  index2  = B3                    ' loop counter


' -----[ Initialization ]---------------------------------------------
'
	BSAVE                           ' save EEPROM image file
	EEPROM ($0E,$1F,$1C,$18)        ' char 0 top
	EEPROM ($1C,$1F,$0E,$00)        ' char 0 bottom
	EEPROM ($0E,$1F,$1F,$18)        ' char 1 top
	EEPROM ($1F,$1F,$0E,$00)        ' char 1 bottom
	EEPROM ($0E,$1F,$1F,$1F)        ' char 2 top
	EEPROM ($1F,$1F,$0E,$00)        ' char 2 bottom
	EEPROM ("THE BASIC STAMP!")     ' display string

	Pins = %00000000                ' clear the pins
	Dirs = %00111111                ' set 0-5 as outputs
	PAUSE 20                        ' let the LCD settle


' Initialize the LCD (Hitatchi HD44780 controller)
'
LCDini: Pins = %00000011                ' 8-bit mode
	PULSOUT E, 1
	PAUSE 5
	PULSOUT E, 1
	PULSOUT E, 1
	Pins = %00000010                ' 4-bit mode
	PULSOUT E, 1
	char = %00001100                ' disp on, crsr off, blink off
	GOSUB WrLCD                     
	char = %00000110                ' inc crsr, no disp shift
	GOSUB WrLCD
	char = %00000001                ' clear LCD
	GOSUB WrLCD
	HIGH RS                         ' LCD to character mode

	char = %01000000                ' address 0 of CG RAM
	GOSUB LCDcmd                    ' prepare to write CG data
	FOR index1 = 0 TO 23            ' build 3 custom chars
	  READ index1, char             ' get byte from EEPROM
	  GOSUB WrLCD                   ' put into LCD CG RAM
	NEXT index1


' -----[ Main Code ]--------------------------------------------------
'
Start:  char = ClrLCD                   ' clear the LCD
	GOSUB LCDcmd

	FOR index1 = 24 TO 39           ' write the character string
	  READ index1,char              ' get char from EEPROM
	  GOSUB WrLCD                   ' write it
	  PAUSE 50                      ' delay 50 ms--for fun only
	NEXT index1
	PAUSE 1000                      ' pause 1 second

	FOR index1 = 0 TO 15            ' cover 16 characters
	  FOR index2 = 0 TO 4           ' 5 characters for animation
	    char = %10000000 | index1   ' set DD RAM address
	    GOSUB LCDcmd                ' move cursor to new addr
	    LOOKUP index2,(0,1,2,1," "),char
	    GOSUB WrLCD                 ' write animation character
	    PAUSE 75                    ' delay between chars
	  NEXT index2
	NEXT index1
	PAUSE 1000

	GOTO Start                      ' replay

	
' -----[ Subroutines ]------------------------------------------------
'

' Send command to the LCD
'
' Load char with command value, then call
'
'   Clear the LCD............. $01, %00000001
'   Home the cursor........... $02, %00000010
'   Display control........... (see below)
'   Entry mode................ (see below)
'   Cursor left............... $10, %00010000
'   Cursor right.............. $14, %00010100
'   Scroll display left....... $18, %00011000
'   Scroll display right...... $1C, %00011100
'   Set CG RAM address........      %01aaaaaa (Character Generator)
'   Set DD RAM address........      %1aaaaaaa (Display Data)
'
' Display control byte:
'
'   % 0 0 0 0 1 D C B
'                  blink character under cursor (1=blink)
'                 cursor on/off (1=on)
'                display on/off (1=on)
'
' Entry mode byte:
'
'   % 0 0 0 0 0 1 X S
'                   shift display (S=1), left (X=1), right (X=0)
'                  cursor move: right (X=1), left (X=0)
'
'
LCDcmd: LOW RS
	GOSUB WrLCD
	HIGH RS
	RETURN


' Write ASCII char to LCD 
'
WrLCD:  Pins = Pins & %00010000         ' RS = 1, data bus clear
	outp = char / 16                ' get high nibble
	Pins = Pins | outp              ' output the nibble
	PULSOUT E, 1                    ' strobe the Enable line
	Pins = Pins & %00010000
	outp = char & %00001111         ' get low nibble
	Pins = Pins | outp
	PULSOUT E, 1
	RETURN

